import 'package:flutter/material.dart';
import 'package:flutter_riverpod/flutter_riverpod.dart';
import 'package:flutter_gen/gen_l10n/app_localizations.dart';
import 'package:gui_app/const/colors.dart';
import 'package:gui_app/const/dimens.dart';
import 'package:gui_app/provider.dart';
import 'package:gui_app/widget/chart.dart';
import 'package:gui_app/widget/energy.dart';
import 'package:gui_app/widget/gauge_target.dart';
import 'package:gui_app/widget/bar.dart';
import 'package:gui_app/widget/log.dart';
import 'package:gui_app/widget/map.dart';
import 'package:gui_app/widget/run_alert.dart';
import 'package:gui_app/widget/slider.dart';
import 'package:gui_app/widget/toggle.dart';
import 'package:gui_app/widget/video.dart';

void main() {
  WidgetsFlutterBinding.ensureInitialized();
  runApp(const ProviderScope(child: MyApp()));
}

class MyApp extends ConsumerWidget {
  const MyApp({Key? key}) : super(key: key);

  @override
  Widget build(BuildContext context, WidgetRef ref) {
    return MaterialApp(
      debugShowCheckedModeBanner: false,
      localizationsDelegates: L10n.localizationsDelegates,
      supportedLocales: L10n.supportedLocales,
      locale: ref.watch(localeProvider),
      theme: ThemeData(
        primarySwatch: Colors.blue,
        fontFamily: 'IPAGothic',
      ),
      home: MyHomePage(),
    );
  }
}

class MyHomePage extends StatelessWidget {
  const MyHomePage({Key? key}) : super(key: key);

  @override
  Widget build(BuildContext context) {
    const List<String> langText = ["JP", "EN"];
    final double deviceWidth = MediaQuery.of(context).size.width;
    final double deviceHeight = MediaQuery.of(context).size.height;
    return Scaffold(
      extendBodyBehindAppBar: true,
      backgroundColor: bodyBackgroundColor,
      body: Column(
        children: [
          // Title and language selection widget
          Container(
            decoration: BoxDecoration(color: headerBackgroundColor),
            height: deviceHeight * titleHeightRatio,
            child: Row(
              children: [
                Expanded(
                  child: Container(
                    padding: EdgeInsets.only(left: titleLeftPadding),
                    child: Text(
                      "Armadillo-IoT G4 Flutter Demo",
                      style: TextStyle(
                          fontSize: deviceWidth * titleFontSizeRatio,
                          color: textColor),
                    ),
                  ),
                ),
                Container(
                  padding: EdgeInsets.only(right: langToggleRightPadding),
                  child: Toggle(
                    names: langText,
                  ),
                ),
              ],
            ),
          ),

          Expanded(
            child: Container(
              margin: const EdgeInsets.all(outsidePadding),
              child: Row(
                children: [
                  // left column widget
                  SizedBox(
                    width: deviceWidth * leftWidthRatio,
                    child: Column(
                      crossAxisAlignment: CrossAxisAlignment.start,
                      children: [
                        RunAlert(height: deviceHeight * RunAlertHeightRatio),
                        Spacer(),
                        EnergyBox(
                          height: deviceHeight * energyBoxHeightRatio,
                        ),
                        Spacer(),
                        Charts(
                          height: deviceHeight * chartsHeightRatio,
                        ),
                        Spacer(),
                        Log(
                          height: deviceHeight * logHeightRatio,
                        ),
                      ],
                    ),
                  ),

                  // keep space in a row
                  Spacer(),
                  // center column widget
                  SizedBox(
                    width: deviceWidth * middleWidthRatio,
                    child: Column(
                      children: [
                        GaugeTarget(
                          height: deviceHeight * gaugeTargetHeightRatio,
                        ),
                        TouchSlider(
                          height: deviceHeight * touchSliderHeightRatio,
                        ),
                        Expanded(
                          child: const Bar(),
                        ),
                      ],
                    ),
                  ),

                  // keep space in a row
                  Spacer(),
                  // right column widget
                  SizedBox(
                    width: deviceWidth * rightWidthRatio,
                    child: Column(
                      crossAxisAlignment: CrossAxisAlignment.end,
                      children: [
                        Expanded(
                          child: SizedBox(
                            child: const OfflineMap(),
                          ),
                        ),
                        Expanded(
                          child: Container(
                            child: const Video(),
                          ),
                        ),
                      ],
                    ),
                  ),
                ],
              ),
            ),
          ),
        ],
      ),
    );
  }
}
