import 'package:gui_app/const/colors.dart';
import 'package:fl_chart/fl_chart.dart';
import 'package:flutter_riverpod/flutter_riverpod.dart';
import 'package:intl/intl.dart';
import 'package:gui_app/const/config.dart';
import 'package:flutter_gen/gen_l10n/app_localizations.dart';
import 'dart:math';

import 'package:flutter/material.dart';

var isAlert = false;
var isPreAlert = false;
var isAlertSwitch = false;
var alertFillColor = bodyBackgroundColor;
var alertLog = [];
var totalCapacityPlotData = <FlSpot>[];
var targetPlotData = <FlSpot>[];
var energyGeneratedPlotData = <FlSpot>[];

final toggleProvider = StateProvider((ref) => [true, false]);
final sliderProvider = StateProvider((ref) => 0);
final timerProvider = StateProvider((ref) => 0);
final localeProvider = StateProvider((ref) => L10n.supportedLocales.last);

final alertProvider = StateProvider((ref) {
  // Change the value of alertProvider only when the target value exceeds
  // the amount of electricity generated.
  if (ref.watch(targetProvider) > ref.watch(energyGeneratedProvider)) {
    isAlert = true;
  } else {
    isAlert = false;
  }
  return isAlert;
});

final alertSwitchProvider = StateProvider((ref) {
  if (ref.watch(alertProvider) && !isPreAlert) {
    isAlertSwitch = !isAlertSwitch;
  }
  isPreAlert = isAlert;
  return isAlertSwitch;
});

final alertBlinkingProvider = StateProvider((ref) {
  ref.watch(timerProvider);
  if (ref.read(alertProvider)) {
    if (alertFillColor == alertBackgroundColor) {
      alertFillColor = bodyBackgroundColor;
    } else {
      alertFillColor = alertBackgroundColor;
    }
  } else {
    alertFillColor = bodyBackgroundColor;
  }
  return alertFillColor;
});

final alertLogProvider = StateProvider((ref) {
  // change value of alertLogProvider when alertProvider changed
  ref.watch(alertSwitchProvider.select((alert) => alert));

  final DateTime now = DateTime.now();
  DateFormat outputFormat = DateFormat('yyyy-MM-dd H:mm:ss');
  alertLog.add(outputFormat.format(now));
  // zeroth array contains the value at initialization
  if (alertLog.length > maxLogNum + 1) {
    alertLog.removeAt(1);
  }
  return alertLog.sublist(1);
});

final targetProvider = Provider((ref) {
  return totalCapacityData * ref.watch(sliderProvider).toDouble() / 100;
});

final energyGeneratedProvider = StateProvider((ref) {
  return (totalCapacityData * energyGeneratedRate +
          Random().nextInt(energyGeneratedRandomMax) +
          sin(ref.watch(timerProvider)) * totalCapacityAmplitude)
      .toInt();
});

final totalCapacityChartProvider = StateProvider((ref) {
  totalCapacityPlotData.add(FlSpot(
      ref.watch(timerProvider).toDouble(), totalCapacityData.toDouble()));
  if (totalCapacityPlotData.length > maxPlotPoint) {
    totalCapacityPlotData.removeAt(0);
  }
  return totalCapacityPlotData;
});

final targetChartProvider = StateProvider((ref) {
  targetPlotData.add(
      FlSpot(ref.watch(timerProvider).toDouble(), ref.read(targetProvider)));
  if (targetPlotData.length > maxPlotPoint) {
    targetPlotData.removeAt(0);
  }
  return targetPlotData;
});

final energyGeneratedChartProvider = StateProvider((ref) {
  energyGeneratedPlotData.add(FlSpot(ref.watch(timerProvider).toDouble(),
      ref.read(energyGeneratedProvider).toDouble()));
  if (energyGeneratedPlotData.length > maxPlotPoint) {
    energyGeneratedPlotData.removeAt(0);
  }
  return energyGeneratedPlotData;
});

final barProvider = StateProvider((ref) {
  final double barData;
  barData = baseTemperature + sin(ref.watch(timerProvider)) * 2;
  return barData;
});
